// Swig instructions to implement the hex_print extension module

%module hex_print
%include typemaps.i

%{
#define NPY_NO_DEPRECATED_API    NPY_1_7_API_VERSION

// Temporary fix for problems with -fvisibility=hidden and octave headers.
#ifdef OCTAVE_EXPORT
  #if defined ( __GNUC__ ) && __GNUC__ > 3
    #undef OCTAVE_EXPORT
    #define OCTAVE_EXPORT    __attribute__ ( ( visibility( "default" ) ) )
  #endif
#endif

  // Print out hex digits of null-terminated C character string.
  void hex_print( const char * string )
  {
    size_t i;
    for(i=0; i<= strlen(string); i++)
      {
	// printf will promote all integers smaller than int to int, leading to sign
	// bit extension for what starts as a signed char, see
	// < https://stackoverflow.com/questions/8060170/printing-hexadecimal-characters-in-c>.
	// Use the cast to avoid that sign extension.
	printf( "%2x ", (unsigned char) string[i]);
      }
    printf( "\n%s\n", string);
  }

  // Print out hex digits of array of null-terminated C character strings.
  void hex_print2( int n, const char ** strings )
  {
    size_t i, j;
    for(j=0; j<n; j++)
      {
	for(i=0; i<= strlen(strings[j]); i++)
	  {
	    // printf will promote all integers smaller than int to int, leading to sign
	    // bit extension for what starts as a signed char, see
	    // < https://stackoverflow.com/questions/8060170/printing-hexadecimal-characters-in-c>.
	    // Use the cast to avoid that sign extension.
	    printf( "%2x ", (unsigned char) strings[j][i]);
	  }
	printf( "\n%s\n", strings[j]);
      }
  }
%}

// typemaps
%include <typemaps.i>

%{
// I hate global variables but this is the best way I can think of
// to manage consistency checking among function arguments.
    static int Alen = 0;
%}

%{
// Convenience functions copied from matwrap-based approach (currently
// stored in bindings/octave/matwrap/wrap_octave.pl) to take care of the
// tricky scalar case and also adopted so that the resulting
// swig-generated source code will look similar to the matwrap-generated
// source code.

    inline int max( int a, int b )
    {
        return a >= b ? a : b;
    }
//
// Function to get the number of dimensions of an object.
//
    static int
    _n_dims( const octave_value &o_obj )
    {
        if ( max( o_obj.columns(), 1 ) > 1 )
            return 2;
        // max is necessary because sometimes
        // rows() or columns() return -1 or 0 for
        // scalars.
        else if ( max( o_obj.rows(), 1 ) > 1 )
            return 1;
        else
            return 0;
    }

//
// Return the n'th dimension of an object.  Dimension 0 is the 1st dimension.
//
    static inline int
    _dim( const octave_value &o_obj, int dim_idx )
    {
        if ( dim_idx == 0 )
            return max( o_obj.rows(), 0 );
        // max is necessary because sometimes
        // rows() or columns() return -1 or 0 for
        // scalars.
        else if ( dim_idx == 1 )
            return max( o_obj.columns(), 0 );
        else
            return 1;
    }

%}

// With count which is stored in Alen to allow possible dimension
// consistency checking for other arguments.
%typemap ( in ) ( int n, const char ** strings )
{
    charMatrix  temp_matrix;
    Cell        temp_cell;
    char        *tmp_cstring;
    std::string str;
    size_t      max_length = 0, non_blank_length;
    int         i, ifcell;
    if ( _n_dims( $input ) > 2 )
    {
        error( "argument must be a scalar or vector or matrix" ); SWIG_fail;
    }
%#if SWIG_OCTAVE_PREREQ( 4, 4, 0 )
    if ( !$input.isempty() )
%#else
    if ( !$input.is_empty() )
%#endif
    {
        Alen = _dim( $input, 0 );
        $1   = Alen;
        $2   = new char*[Alen];
%#if SWIG_OCTAVE_PREREQ( 4, 4, 0 )
        ifcell = $input.iscell();
%#else
        ifcell = $input.is_cell();
%#endif
        if ( ifcell )
        {
            temp_cell = $input.cell_value();
        }
        else
        {
            temp_matrix = $input.char_matrix_value();
            // Allow one extra space for null termination.
            max_length = _dim( $input, 1 ) + 1;
        }

        for ( i = 0; i < Alen; i++ )
        {
            // Must copy string to "permanent" location because the string
            // location corresponding to tmp_cstring gets
            // overwritten for each iteration of loop.
            if ( ifcell )
            {
                if ( temp_cell.elem( i ).is_string() )
                {
                    str = temp_cell.elem( i ).string_value();
                    // leave room for null termination.
                    max_length  = str.size() + 1;
                    tmp_cstring = (char *) str.c_str();
                }
                else
                {
                    // Use null string if user attempts to pass a cell array
                    // with a non-string element (likely an empty element
                    // since that should be allowed by the interface
                    // if that element is going to be unused).
                    // leave room for null termination.
                    max_length  = 1;
                    tmp_cstring = (char *) "";
                }
            }
            else
            {
	      tmp_cstring = NULL;
	      tmp_cstring = (char *) temp_matrix.row_as_string( i ).c_str();
	      if( !tmp_cstring )
		{
		  // For this failure adjust Alen to only delete previously defined $2[i].
		  Alen = i;
		  error( "charMatrix row_as_string( i ).c_str() method failed" );
		  SWIG_fail;
		}
            }
            $2[i] = new char[max_length];
            strncpy( $2[i], tmp_cstring, max_length - 1 );
            $2[i][max_length - 1] = '\0';
            // All the trailing blank crapola should not be needed for
            // string cell arrays.
            if ( !ifcell )
            {
                // remove trailing-blank padding that is used by the
                // charMatrix class to insure all strings in a given
                // charMatrix instance have the same length.
                // This transformation also removes legitimate trailing
                // blanks but there is nothing we can do about that
                // for the charMatrix class.

                // Look for trailing nulls first (just in case, although that
                // shouldn't happen if charMatrix implemented as documented)
                // before looking for trailing blanks.
                non_blank_length = max_length - 2;
                while ( non_blank_length >= 0 && $2[i][non_blank_length] == '\0' )
                {
                    non_blank_length--;
                }
                while ( non_blank_length >= 0 && $2[i][non_blank_length] == ' ' )
                {
                    non_blank_length--;
                }
                $2[i][non_blank_length + 1] = '\0';
            }
        }
    }
    else
    {
        $1 = 0;
        $2 = NULL;
    }
}
%typemap ( freearg ) ( int n, const char ** strings )
{
    int i;
    if ( $2 != NULL )
    {
        for ( i = 0; i < Alen; i++ )
        {
            delete[] $2[i];
        }
        delete[] $2;
    }
}

// Interface the above routines with swig
void hex_print( const char * string );
void hex_print2( int n, const char ** strings);

